@@ -69,8 +69,7 @@ mk_profile_dir()
     ENTER;
     sprintf(fname, "%s/.fbpanel", getenv("HOME"));
     if ((ret = stat(fname, &buf))) {
-        if (verbose)
-            ERR("creating %s\n", fname);
+        LOG(LOG_INFO, "creating %s\n", fname);
         mkdir(fname, 0755);
         ret = stat(fname, &buf);
     }
@@ -95,20 +94,16 @@ response_event(GtkDialog *widget, gint arg1, gpointer user_data)
         gtk_widget_destroy(dialog);
         dialog = NULL;
         break;
-    case GTK_RESPONSE_CANCEL:
+    case GTK_RESPONSE_CLOSE:
         gtk_widget_hide(dialog);
         break;
-    case GTK_RESPONSE_OK:
-        config = 0;
-        gtk_widget_hide(dialog);
     case GTK_RESPONSE_APPLY:
         if (!mk_profile_dir()) {
             ERR("can't make ~/.fbpanel direcory\n");
             RET();
         }
         sprintf(fname, "%s/.fbpanel/%s", getenv("HOME"), cprofile);
-        if (verbose)
-            ERR("saving profile %s to file %s\n", cprofile, fname);
+        LOG(LOG_WARN, "saving profile %s as %s\n", cprofile, fname);
         if (!(fp = fopen(fname, "w"))) {
             ERR("can't open for write %s:", fname);
             perror(NULL);
@@ -126,25 +121,25 @@ response_event(GtkDialog *widget, gint arg1, gpointer user_data)
 }
 
 static void
-set_edge(GtkWidget *item, gpointer bp)
+set_edge(GtkComboBox *widget, gpointer bp)
 {
     int edge;
     
     ENTER;    
-    edge = gtk_option_menu_get_history(GTK_OPTION_MENU(item)) + 1;
+    edge = gtk_combo_box_get_active(widget) + 1;
     DBG("edge=%d\n", edge);    
     RET();
 }
 
 
 static void
-set_allign(GtkWidget *item, gpointer bp)
+set_allign(GtkComboBox *widget, gpointer bp)
 {
     int allign;
     gboolean t;
     
     ENTER;	
-    allign = gtk_option_menu_get_history(GTK_OPTION_MENU(item)) + 1;
+    allign = gtk_combo_box_get_active(widget) + 1;
     DBG("allign=%d\n", allign);
     t = (allign != ALLIGN_CENTER);
     gtk_widget_set_sensitive(margin_label, t);
@@ -156,7 +151,7 @@ set_allign(GtkWidget *item, gpointer bp)
 GtkWidget *
 mk_position()
 {
-    GtkWidget *hbox, *hbox2, *label, *frame, *mi, *menu;
+    GtkWidget *hbox, *hbox2, *label, *frame;
     GtkWidget *t;
    
    
@@ -188,29 +183,15 @@ mk_position()
     gtk_table_attach(GTK_TABLE(t), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
     gtk_size_group_add_widget(sg, label);
     
-    menu = gtk_menu_new();
-    
-    mi = gtk_menu_item_new_with_label("Left");
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-
-    mi = gtk_menu_item_new_with_label("Right");
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-
-    mi = gtk_menu_item_new_with_label("Top");
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-
-    mi = gtk_menu_item_new_with_label("Bottom");
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-  
-    edge_opt = gtk_option_menu_new();
-    gtk_widget_show(edge_opt);
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(edge_opt), menu);
+    edge_opt = gtk_combo_box_new_text();
+    gtk_combo_box_append_text(GTK_COMBO_BOX(edge_opt), "Left");
+    gtk_combo_box_append_text(GTK_COMBO_BOX(edge_opt), "Right");
+    gtk_combo_box_append_text(GTK_COMBO_BOX(edge_opt), "Top");
+    gtk_combo_box_append_text(GTK_COMBO_BOX(edge_opt), "Bottom");
     g_signal_connect(G_OBJECT(edge_opt), "changed", G_CALLBACK(set_edge), NULL);
-
+    
+    gtk_widget_show(edge_opt);
+  
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX (hbox), edge_opt, FALSE, TRUE, 0);
     gtk_table_attach(GTK_TABLE(t), hbox, 1, 2, 0, 1, GTK_FILL, 0, 0, 0);
@@ -224,24 +205,12 @@ mk_position()
     
     gtk_table_attach(GTK_TABLE(t), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
         
-    menu = gtk_menu_new();
-			
-    mi = gtk_menu_item_new_with_label("Left");
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-
-    mi = gtk_menu_item_new_with_label("Center");
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-
-    mi = gtk_menu_item_new_with_label("Right");
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-
-    allign_opt = gtk_option_menu_new();
-    gtk_widget_show(allign_opt);
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(allign_opt), menu);
+    allign_opt = gtk_combo_box_new_text();
+    gtk_combo_box_append_text(GTK_COMBO_BOX(allign_opt), "Left");
+    gtk_combo_box_append_text(GTK_COMBO_BOX(allign_opt), "Center");
+    gtk_combo_box_append_text(GTK_COMBO_BOX(allign_opt), "Right");
     g_signal_connect(G_OBJECT(allign_opt), "changed", G_CALLBACK(set_allign), NULL);
+    gtk_widget_show(allign_opt);
 
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX (hbox), allign_opt, FALSE, TRUE, 0);
@@ -273,7 +242,7 @@ set_width(GtkWidget *item, gpointer bp)
     
     ENTER;
     
-    widthtype = gtk_option_menu_get_history(GTK_OPTION_MENU(item)) + 1;
+    widthtype = gtk_combo_box_get_active(GTK_COMBO_BOX(item)) + 1;
     DBG("widthtype=%d\n", widthtype);
     t = (widthtype != WIDTH_REQUEST);
     gtk_widget_set_sensitive(width_spinb, t);
@@ -296,7 +265,7 @@ set_width(GtkWidget *item, gpointer bp)
 GtkWidget *
 mk_size()
 {
-    GtkWidget *hbox, *hbox2, *label, *frame, *mi, *menu;
+    GtkWidget *hbox, *hbox2, *label, *frame;
     GtkWidget *t;
   
    
@@ -332,25 +301,15 @@ mk_size()
     width_spinb = gtk_spin_button_new (width_adj, 1.0, 0);
     gtk_table_attach(GTK_TABLE(t), width_spinb, 1, 2, 0, 1, GTK_FILL, 0, 0, 0);
     
-    menu = gtk_menu_new();
-    
-    mi = gtk_menu_item_new_with_label("dynamic");
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
 
-    mi = gtk_menu_item_new_with_label("pixels");
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-
-    mi = gtk_menu_item_new_with_label("% of edge");
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-
-    width_opt = gtk_option_menu_new();
-    gtk_widget_show(width_opt);
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(width_opt), menu);
+    width_opt = gtk_combo_box_new_text();
+    gtk_combo_box_append_text(GTK_COMBO_BOX(width_opt), "dynamic");
+    gtk_combo_box_append_text(GTK_COMBO_BOX(width_opt), "pixels");
+    gtk_combo_box_append_text(GTK_COMBO_BOX(width_opt), "% of edge");
     g_signal_connect(G_OBJECT(width_opt), "changed", G_CALLBACK(set_width), NULL);
+    gtk_widget_show(width_opt);
 
+    
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX (hbox), width_opt, FALSE, TRUE, 0);
     gtk_table_attach(GTK_TABLE(t), hbox, 2, 3, 0, 1, GTK_FILL, 0, 0, 0);
@@ -368,16 +327,11 @@ mk_size()
     height_spinb = gtk_spin_button_new (height_adj, 1.0, 0);
     gtk_table_attach(GTK_TABLE(t), height_spinb, 1, 2, 1, 2, GTK_FILL, 0, 0, 0);
     
-    menu = gtk_menu_new();
-			
-    mi = gtk_menu_item_new_with_label("pixels");
-    g_object_set_data(G_OBJECT(mi), "user-data", GUINT_TO_POINTER(HEIGHT_PIXEL));
-    gtk_widget_show(mi);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-
-    height_opt = gtk_option_menu_new();
+    
+    height_opt = gtk_combo_box_new_text();
+    gtk_combo_box_append_text(GTK_COMBO_BOX(height_opt), "pixels");
+    //g_signal_connect(G_OBJECT(height_opt), "changed", G_CALLBACK(set_height), NULL);
     gtk_widget_show(height_opt);
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(height_opt), menu);
 
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX (hbox), height_opt, FALSE, TRUE, 0);
@@ -494,41 +448,39 @@ dialog_delete_event( GtkWidget *widget, GdkEvent  *event, gpointer   data )
     RET(FALSE);
 }
 
+static GtkWidget *
+mk_tab_plugins()
+{
+    GtkWidget *sw, *paned, *hbox, *label;
+
+    paned = gtk_vpaned_new();
+
+    hbox = gtk_hbox_new(FALSE, 0);
+    label = gtk_label_new("Right-click to get context menu. Drag & Drop to change order.");
+    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+    gtk_box_pack_end(GTK_BOX(hbox), label, TRUE, TRUE, 5);
+
+    gtk_paned_add1(GTK_PANED(paned), hbox);
+
+    sw = gtk_label_new("Plugins...");
+    gtk_paned_add2(GTK_PANED(paned), sw);
+    
+    RET(paned);
+}
 
 static GtkWidget *
-mk_dialog()
+mk_tab_general()
 {
-    GtkWidget *sw, *frame, *page;
-           
-    DBG("creating dialog\n");
-    dialog = gtk_dialog_new_with_buttons ("fbpanel configurator",
-          NULL,
-          0, //GTK_DIALOG_DESTROY_WITH_PARENT,
-          GTK_STOCK_APPLY,
-          GTK_RESPONSE_APPLY,
-          GTK_STOCK_OK,
-          GTK_RESPONSE_OK,
-          GTK_STOCK_CANCEL,
-          GTK_RESPONSE_CANCEL,
-          NULL);
-    DBG("connecting sugnal to %p\n",  dialog);
-    g_signal_connect (G_OBJECT(dialog), "response",     (GCallback) response_event,       NULL);
-    g_signal_connect (G_OBJECT(dialog), "destroy",      (GCallback) dialog_destroy_event, NULL);
-    g_signal_connect (G_OBJECT(dialog), "delete_event", (GCallback) dialog_delete_event,  NULL);
-    gtk_window_set_modal(GTK_WINDOW(dialog), FALSE);
-    gtk_window_set_default_size(GTK_WINDOW(dialog), 300, 400);
+    GtkWidget *frame, *page;
 
-    //gtk_window_set_skip_taskbar_hint(GTK_WINDOW(dialog), TRUE);
-    //gtk_window_set_skip_pager_hint(GTK_WINDOW(dialog), TRUE);
-
-    
+    /*
     sw = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), sw);
-    
+    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_NONE);
+    gtk_container_set_border_width(GTK_CONTAINER(sw), 0);
+    */
     page = gtk_vbox_new(FALSE, 1);
-    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW (sw), page);
-    
+        
     sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
     
     //position 
@@ -547,6 +499,50 @@ mk_dialog()
     //properties 
     frame = mk_properties();
     gtk_box_pack_start(GTK_BOX (page), frame, FALSE, TRUE, 0);
+    /*
+    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW (sw), page);
+    */
+    RET(page);
+}
+
+
+static GtkWidget *
+mk_dialog()
+{
+    GtkWidget *sw, *nb, *label;
+           
+    DBG("creating dialog\n");
+    dialog = gtk_dialog_new_with_buttons ("fbpanel configurator",
+          NULL,
+          0, //GTK_DIALOG_DESTROY_WITH_PARENT,
+          GTK_STOCK_APPLY,
+          GTK_RESPONSE_APPLY,
+          GTK_STOCK_CLOSE,
+          GTK_RESPONSE_CLOSE,
+          NULL);
+    DBG("connecting sugnal to %p\n",  dialog);
+    g_signal_connect (G_OBJECT(dialog), "response",     (GCallback) response_event,       NULL);
+    g_signal_connect (G_OBJECT(dialog), "destroy",      (GCallback) dialog_destroy_event, NULL);
+    g_signal_connect (G_OBJECT(dialog), "delete_event", (GCallback) dialog_delete_event,  NULL);
+    gtk_window_set_modal(GTK_WINDOW(dialog), FALSE);
+    gtk_window_set_default_size(GTK_WINDOW(dialog), 400, 500);
+
+    //gtk_window_set_skip_taskbar_hint(GTK_WINDOW(dialog), TRUE);
+    //gtk_window_set_skip_pager_hint(GTK_WINDOW(dialog), TRUE);
+      
+    nb = gtk_notebook_new();
+    gtk_notebook_set_show_border (GTK_NOTEBOOK(nb), FALSE);
+    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), nb);
+
+    sw = mk_tab_general();
+    label = gtk_label_new("General");
+    gtk_misc_set_padding(GTK_MISC(label), 4, 1);
+    gtk_notebook_append_page(GTK_NOTEBOOK(nb), sw, label);
+
+    sw = mk_tab_plugins();
+    label = gtk_label_new("Plugins");
+    gtk_misc_set_padding(GTK_MISC(label), 4, 1);
+    gtk_notebook_append_page(GTK_NOTEBOOK(nb), sw, label);
     
     g_object_unref(sg);
     
@@ -564,12 +560,12 @@ update_opt_menu(GtkWidget *w, int ind)
     ENTER;
     /* this trick will trigger "changed" signal even if active entry is
      * not actually changing */
-    i = gtk_option_menu_get_history(GTK_OPTION_MENU(w));
+    i = gtk_combo_box_get_active(GTK_COMBO_BOX(w));
     if (i == ind) {
         i = i ? 0 : 1;
-        gtk_option_menu_set_history(GTK_OPTION_MENU(w), i);
+        gtk_combo_box_set_active(GTK_COMBO_BOX(w), i);
     }
-    gtk_option_menu_set_history(GTK_OPTION_MENU(w), ind);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(w), ind);
     RET();
 }
 
@@ -609,6 +605,7 @@ configure(void)
     
     update_opt_menu(width_opt, p->widthtype - 1);
     gtk_adjustment_set_value(width_adj, p->width);
+    update_opt_menu(height_opt, HEIGHT_PIXEL - 1);
     gtk_adjustment_set_value(height_adj, p->height);
 
     update_toggle_button(tr_checkb, p->transparent);
@@ -631,12 +628,12 @@ global_config_save(FILE *fp)
     fprintf(fp, "\n\n");
     fprintf(fp, "Global {\n");
     fprintf(fp, "    edge = %s\n",
-          num2str(edge_pair, gtk_option_menu_get_history(GTK_OPTION_MENU(edge_opt)) + 1, "none"));
+          num2str(edge_pair, gtk_combo_box_get_active(GTK_COMBO_BOX(edge_opt)) + 1, "none"));
     fprintf(fp, "    allign = %s\n",
-          num2str(allign_pair, gtk_option_menu_get_history(GTK_OPTION_MENU(allign_opt)) + 1, "none"));
+          num2str(allign_pair, gtk_combo_box_get_active(GTK_COMBO_BOX(allign_opt)) + 1, "none"));
     fprintf(fp, "    margin = %d\n", (int) margin_adj->value);
     fprintf(fp, "    widthtype = %s\n",
-          num2str(width_pair, gtk_option_menu_get_history(GTK_OPTION_MENU(width_opt)) + 1, "none"));
+          num2str(width_pair, gtk_combo_box_get_active(GTK_COMBO_BOX(width_opt)) + 1, "none"));
     fprintf(fp, "    width = %d\n", (int) width_adj->value);
     fprintf(fp, "    height = %d\n", (int) height_adj->value);
     fprintf(fp, "    transparent = %s\n",
