@@ -13,6 +13,7 @@
 
 #include "misc.h"
 #include "panel.h"
+#include "gtkbgbox.h"
 
 //#define DEBUG
 #include "dbg.h"
@@ -290,7 +291,6 @@ get_xaproperty (Window win, Atom prop, Atom type, int *nitems)
     unsigned long items_ret;
     unsigned long after_ret;
     unsigned char *prop_data;
-    int i;
     
     ENTER;
     prop_data = NULL;
@@ -773,3 +773,120 @@ guint32 gcolor2rgb24(GdkColor *color)
     RET(i);
 }
 
+
+static gboolean
+fb_button_enter (GtkImage *widget, GdkEventCrossing *event)
+{
+    GdkPixbuf *dark, *light;
+    int i;
+    guint hicolor;
+    guchar *src, *up, extra[3];
+ 
+    ENTER;
+    if (gtk_image_get_storage_type(widget) != GTK_IMAGE_PIXBUF)
+        RET(TRUE);
+    light = g_object_get_data(G_OBJECT(widget), "light");
+    dark = gtk_image_get_pixbuf(widget);
+    if (!light) {
+        hicolor = (gint) g_object_get_data(G_OBJECT(widget), "hicolor");
+        light = gdk_pixbuf_add_alpha(dark, FALSE, 0, 0, 0);      
+        src = gdk_pixbuf_get_pixels (light);
+        for (i = 2; i >= 0; i--, hicolor >>= 8)
+            extra[i] = hicolor & 0xFF;
+        for (up = src + gdk_pixbuf_get_height(light) * gdk_pixbuf_get_rowstride (light);
+             src < up; src+=4) {
+            if (src[3] == 0)
+                continue;
+            for (i = 0; i < 3; i++) {
+                if (src[i] + extra[i] >= 255)
+                    src[i] = 255;
+                else
+                    src[i] += extra[i];
+            }
+        }
+        if (!light)
+            RET(TRUE);
+        g_object_set_data_full (G_OBJECT(widget), "light", light, g_object_unref);
+    }
+    g_object_ref(dark);
+    g_object_set_data_full (G_OBJECT(widget), "dark", dark, g_object_unref);
+    gtk_image_set_from_pixbuf(widget, light);
+    RET(TRUE);
+    
+}
+
+static gboolean
+fb_button_leave (GtkImage *widget, GdkEventCrossing *event, gpointer user_data)
+{
+    GdkPixbuf *dark;
+    
+    ENTER;
+    if (gtk_image_get_storage_type(widget) != GTK_IMAGE_PIXBUF)
+        RET(TRUE);
+    dark = g_object_get_data(G_OBJECT(widget), "dark");
+    if (dark)
+        gtk_image_set_from_pixbuf(widget, dark);
+    RET(TRUE);    
+}
+
+
+GtkWidget *
+fb_button_new_from_file(gchar *fname, int width, int height, guint hicolor, gboolean keep_ratio)
+{
+    GtkWidget *b, *image;
+    
+    ENTER;
+    b = gtk_bgbox_new();
+    gtk_container_set_border_width(GTK_CONTAINER(b), 0);
+    GTK_WIDGET_UNSET_FLAGS (b, GTK_CAN_FOCUS);
+    image = gtk_image_new_from_file_scaled(fname, width, height, keep_ratio);
+    g_object_set_data(G_OBJECT(image), "hicolor", (gpointer)hicolor);
+    gtk_misc_set_padding (GTK_MISC(image), 0, 0);
+    if (hicolor > 0) {
+        gtk_widget_add_events(b, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
+        g_signal_connect_swapped (G_OBJECT (b), "enter-notify-event",
+              G_CALLBACK (fb_button_enter), image);
+        g_signal_connect_swapped (G_OBJECT (b), "leave-notify-event",
+              G_CALLBACK (fb_button_leave), image);
+    }
+    gtk_container_add(GTK_CONTAINER(b), image);
+    gtk_widget_show(image);
+    gtk_widget_show(b);
+    RET(b);
+}
+
+GtkWidget *
+fb_button_new_from_file_with_label(gchar *fname, int width, int height,
+      guint hicolor, gboolean keep_ratio, gchar *name)
+{
+    GtkWidget *b, *image, *box, *label;
+    
+    ENTER;
+    b = gtk_bgbox_new();
+    gtk_container_set_border_width(GTK_CONTAINER(b), 0);
+    GTK_WIDGET_UNSET_FLAGS (b, GTK_CAN_FOCUS);
+
+    box = gtk_hbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(box), 0);
+    GTK_WIDGET_UNSET_FLAGS (box, GTK_CAN_FOCUS);
+    gtk_container_add(GTK_CONTAINER(b), box);
+    
+    image = gtk_image_new_from_file_scaled(fname, width, height, keep_ratio);
+    g_object_set_data(G_OBJECT(image), "hicolor", (gpointer)hicolor);
+    gtk_misc_set_padding (GTK_MISC(image), 0, 0);
+    if (hicolor > 0) {
+        gtk_widget_add_events(b, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
+        g_signal_connect_swapped (G_OBJECT (b), "enter-notify-event",
+              G_CALLBACK (fb_button_enter), image);
+        g_signal_connect_swapped (G_OBJECT (b), "leave-notify-event",
+              G_CALLBACK (fb_button_leave), image);
+    }
+    gtk_box_pack_start(GTK_BOX(box), image, FALSE, FALSE, 0);
+    if (name) {
+        label =  gtk_label_new(name);
+        gtk_misc_set_padding(GTK_MISC(label), 2, 0);
+        gtk_box_pack_end(GTK_BOX(box), label, FALSE, FALSE, 0);
+    }
+    gtk_widget_show_all(b);
+    RET(b);
+}
