@@ -51,6 +51,7 @@ typedef struct {
     int setstrut : 1;
     int round_corners : 1;
     int transparent : 1;
+    int spacing;
     
     int desknum;
     int curdesk;
@@ -145,4 +146,6 @@ extern int verbose;
 
 extern FbEv *fbev;
 
+#define FBPANEL_WIN(win)  gdk_window_lookup(win)
+
 #endif
