@@ -273,7 +273,7 @@ do_net_client_list(GtkWidget *widget, icons *ics)
             tk->win = ics->wins[i];
             tk->ics = ics;
             
-            if (!gdk_window_lookup(tk->win))
+            if (!FBPANEL_WIN(tk->win))
                 XSelectInput (GDK_DISPLAY(), tk->win, PropertyChangeMask | StructureNotifyMask); 
             get_wmclass(tk);
             set_icon_maybe(ics, tk);
