@@ -1,11 +1,14 @@
 #include <stdlib.h>
+#include <string.h>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <glib.h>
 
 #include "panel.h"
 #include "misc.h"
 #include "plugin.h"
 #include "bg.h"
+#include "gtkbgbox.h"
 
 //#define DEBUG
 #include "dbg.h"
@@ -22,39 +25,21 @@
 
 typedef struct {
     GtkTooltips *tips;
-    GtkWidget *mainw, *mbar;
+    GtkWidget *menu, *box, *bg, *label;
+    gulong handler_id;
     int iconsize, paneliconsize;
+    GSList *files;
 } menup;
 
-static gchar *menu_rc = "style 'menubar-style'\n"
-"{\n"
-"GtkMenuBar::shadow-type = none\n"
-"GtkMenuBar::internal-padding = 0\n"
-"GtkMenu::vertical-padding = 0\n"
-"}\n"
-"class 'GtkMenuBar' style 'menubar-style'\n"
-"class 'GtkMenu'    style 'menubar-style'\n";
-/*
-static gchar *transparent_rc = "style 'transparent-style'\n"
-"{\n"
-"bg_pixmap[NORMAL] = \"<parent>\"\n"
-"bg_pixmap[INSENSITIVE] = \"<parent>\"\n"
-"bg_pixmap[PRELIGHT] = \"<parent>\"\n"
-"bg_pixmap[SELECTED] = \"<parent>\"\n"
-"bg_pixmap[ACTIVE] = \"<parent>\"\n"
-"}\n"
-"widget '*.mainmenu' style 'transparent-style'\n";
-*/
-
 static void
 menu_destructor(plugin *p)
 {
     menup *m = (menup *)p->priv;
 
     ENTER;
-    gtk_widget_destroy(m->mainw);
-  
-    //
+    g_signal_handler_disconnect(G_OBJECT(m->bg), m->handler_id);
+    gtk_widget_destroy(m->menu);
+    gtk_widget_destroy(m->box);
     g_free(m);
     RET();
 }
@@ -67,7 +52,7 @@ spawn_app(GtkWidget *widget, gpointer data)
     ENTER;    
     if (data) {
         if (! g_spawn_command_line_async(data, &error) ) {
-            ERR("can't spawn %s\nError is %s\n", data, error->message);
+            ERR("can't spawn %s\nError is %s\n", (char *)data, error->message);
             g_error_free (error);
         }
     }
@@ -83,43 +68,86 @@ run_command(GtkWidget *widget, void (*cmd)(void))
     RET();
 }
 
-static FILE *
-read_include(FILE *fp)
+static void
+menu_pos(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, GtkWidget *widget)
 {
-    FILE *fi;
-    line s;
-    gchar *name;
+    int ox, oy, w, h;
+    plugin *p;
     
     ENTER;
-    s.len = 256;
-    name = 0;
-    while (get_line(fp, &s) != LINE_BLOCK_END) {
-        if (s.type == LINE_VAR) {
-            if (!g_ascii_strcasecmp(s.t[0], "name")) 
-                name = expand_tilda(s.t[1]);
-            else {
-                ERR( "menu/include: unknown var %s\n", s.t[0]);
-                goto error;
-            }
-        } 
+    p = g_object_get_data(G_OBJECT(widget), "plugin");
+    gdk_window_get_origin(widget->window, &ox, &oy);
+    w = GTK_WIDGET(menu)->requisition.width;
+    h = GTK_WIDGET(menu)->requisition.height;
+    if (p->panel->orientation == ORIENT_HORIZ) {
+        *x = ox;
+        if (*x + w > gdk_screen_width())
+            *x = ox + widget->allocation.width - w;
+        *y = oy - h;
+        if (*y < 0)
+            *y = oy + widget->allocation.height;
+    } else {
+        *x = ox + widget->allocation.width;
+        if (*x > gdk_screen_width())
+            *x = ox - w;
+        *y = oy;
+        if (*y + h >  gdk_screen_height())
+            *y = oy + widget->allocation.height - h;        
     }
-
-    if ((fi = fopen(name, "r"))) {
-        ERR("Including %s\n", name);
-        g_free(name);
-    }
-    else {
-        ERR("Can't include %s\n", name);
-	goto error;
-    }
-      
-    RET(fi);
-
- error:
-    g_free(name);
-    RET(NULL);
+    DBG("widget: x,y=%d,%d  w,h=%d,%d\n", ox, oy,
+          widget->allocation.width, widget->allocation.height );
+    DBG("w-h %d %d\n", w, h);
+    *push_in = TRUE;
+    RET();
 }
 
+static gboolean
+my_button_pressed(GtkWidget *widget, GdkEventButton *event, GtkMenu *menu)
+{
+    ENTER;  
+    if ((event->type == GDK_BUTTON_PRESS)
+          && (event->x >=0 && event->x < widget->allocation.width)
+          && (event->y >=0 && event->y < widget->allocation.height)) {
+        gtk_menu_popup(menu,
+              NULL, NULL, (GtkMenuPositionFunc)menu_pos, widget, 
+              event->button, event->time);
+    }
+    RET(TRUE);
+}
+
+
+static GtkWidget *
+make_button(plugin *p, gchar *fname, gchar *name, GtkWidget *menu)
+{
+    int w, h;
+    menup *m;
+    
+    ENTER;
+    m = (menup *)p->priv;
+    m->menu = menu;
+    if (p->panel->orientation == ORIENT_HORIZ) {
+        w = 10000;
+        h = p->panel->ah;
+    } else {
+        w = p->panel->aw;
+        h = 10000;
+    }
+    m->bg = fb_button_new_from_file_with_label(fname, w, h, 0xFF0000, TRUE,
+          (p->panel->orientation == ORIENT_HORIZ ? name : NULL));
+    gtk_widget_show(m->bg);  
+    gtk_box_pack_start(GTK_BOX(m->box), m->bg, FALSE, FALSE, 0);
+    if (p->panel->transparent) 
+        gtk_bgbox_set_background(m->bg, BG_ROOT, p->panel->tintcolor, p->panel->alpha);
+    
+
+    m->handler_id = g_signal_connect (G_OBJECT (m->bg), "button-press-event",
+          G_CALLBACK (my_button_pressed), menu);
+    g_object_set_data(G_OBJECT(m->bg), "plugin", p);
+   
+    RET(m->bg);
+}
+   
+
 static GtkWidget *
 read_item(plugin *p)
 {
@@ -163,6 +191,7 @@ read_item(plugin *p)
         g_free(name);
     if (fname) {
         GtkWidget *img;
+        
         img = gtk_image_new_from_file_scaled(fname, m->iconsize, m->iconsize, TRUE);
         gtk_widget_show(img);
         gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), img);
@@ -196,26 +225,54 @@ read_separator(plugin *p)
     RET(gtk_separator_menu_item_new());
 }
 
+static void
+read_include(plugin *p)
+{
+    gchar *name;
+    line s;
+    menup *m = (menup *)p->priv;
+    FILE *fp;
+    
+    ENTER;
+    s.len = 256;
+    name = NULL;
+    while (get_line(p->fp, &s) != LINE_BLOCK_END) {
+        if (s.type == LINE_VAR) {
+            if (!g_ascii_strcasecmp(s.t[0], "name")) 
+                name = expand_tilda(s.t[1]);
+            else  {
+                ERR( "menu/include: unknown var %s\n", s.t[0]);
+                RET();
+            }
+        }
+    }
+    if ((fp = fopen(name, "r"))) {
+        LOG(LOG_INFO, "Including %s\n", name);
+        m->files = g_slist_prepend(m->files, p->fp);
+        p->fp = fp;
+    } else {
+        ERR("Can't include %s\n", name);
+    }
+    if (name) g_free(name);    
+    RET();
+}
+
 static GtkWidget *
-read_submenu(plugin *p, int level)
+read_submenu(plugin *p, gboolean as_item)
 {
     line s;
     GtkWidget *mi, *menu;
-    gchar *name, *fname;
+    gchar name[256], *fname;
     menup *m = (menup *)p->priv;
-    FILE *fbackup;
-    int isinclude;
+
     
     ENTER;
     s.len = 256;
     menu = gtk_menu_new ();
     gtk_container_set_border_width(GTK_CONTAINER(menu), 0);
-    gtk_widget_set_name(GTK_WIDGET(menu), "mainmenu");
-    //gtk_rc_parse_string(menu_rc);
-    
-    name = fname = 0;
-    isinclude = 0;
-    fbackup = 0;
+  
+    fname = 0;
+    name[0] = 0;
     while (get_line(p->fp, &s) != LINE_BLOCK_END) {       
         if (s.type == LINE_BLOCK_START) {
             mi = NULL;
@@ -224,21 +281,10 @@ read_submenu(plugin *p, int level)
             } else if (!g_ascii_strcasecmp(s.t[0], "separator")) {
                 mi = read_separator(p);
             } else if (!g_ascii_strcasecmp(s.t[0], "menu")) {
-                mi = read_submenu(p, level+1);
+                mi = read_submenu(p, TRUE);
             } else if (!g_ascii_strcasecmp(s.t[0], "include")) {
-                if (isinclude)
-		   {
-		     ERR("menu: illegal include in this context\n");
-                     goto error;
-		   }
-                fbackup = p->fp;
-                if ((p->fp = read_include(p->fp))) {
-		    isinclude = 1;
-		    continue;
-		} else {
-                    p->fp = fbackup;
-                    goto error;
-		}
+                read_include(p);
+                continue;
             } else {
                 ERR("menu: unknown block %s\n", s.t[0]);
                 goto error;
@@ -247,51 +293,52 @@ read_submenu(plugin *p, int level)
                 ERR("menu: can't create menu item\n");
                 goto error;
             }
+            gtk_widget_show(mi);
             gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
         } else if (s.type == LINE_VAR) {
             if (!g_ascii_strcasecmp(s.t[0], "image")) 
                 fname = expand_tilda(s.t[1]);
             else if (!g_ascii_strcasecmp(s.t[0], "name"))
-                name = g_strdup(s.t[1]);
-             else {
+                strcpy(name, s.t[1]);
+            else {
                 ERR("menu: unknown var %s\n", s.t[0]);
                 goto error;
             }
-        } else if (s.type == LINE_NONE && isinclude) {
-            isinclude = 0;
-            fclose(p->fp);
-            p->fp = fbackup;
-        } else {
+        } else if (s.type == LINE_NONE) {
+            if (m->files) {
+                fclose(p->fp);
+                p->fp = m->files->data;
+                m->files = g_slist_delete_link(m->files, m->files);
+            }
+        }  else {
             ERR("menu: illegal in this context %s\n", s.str);
             goto error;
         }
     }
-    mi = gtk_image_menu_item_new_with_label(name ? name : "");
-    if (name) 
-        g_free(name);
-    if (fname) {
-        GtkWidget *img;
-
-        if (level == 0) {
-            //bg_set_transparent_background(mi, p->panel);
-            img = gtk_image_new_from_file_scaled(fname,
-                  m->paneliconsize, m->paneliconsize, TRUE);
-            gtk_widget_set_name(mi, "mainmenu");
-        } else
+    if (as_item) {
+        mi = gtk_image_menu_item_new_with_label(name ? name : "");
+        if (fname) {
+            GtkWidget *img;
             img = gtk_image_new_from_file_scaled(fname, m->iconsize, m->iconsize, TRUE);
-        gtk_widget_show(img);
-        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
-        g_free(fname);
+            gtk_widget_show(img);
+            gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+            g_free(fname);
+        }
+        gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), menu);
+        RET(mi);
+    } else {
+        mi = make_button(p, fname, name, menu);
+        if (fname)
+            g_free(fname);
+        RET(mi);
     }
-    gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), menu);
-    RET(mi);
-
+    
  error:
     // FIXME: we need to recursivly destroy all child menus and their items
     gtk_widget_destroy(menu);
     g_free(fname);
     g_free(name);
-    RET(0);
+    RET(NULL);
 }
 
 
@@ -301,45 +348,29 @@ static int
 menu_constructor(plugin *p)
 {
     menup *m;
-    GtkWidget *mi;
-    line s;
 
     ENTER;
-    s.len = 256;
     m = g_new0(menup, 1);
     g_return_val_if_fail(m != NULL, 0);
     p->priv = m;
-  
+
+    //gtk_rc_parse_string(menu_rc);
     if  (p->panel->orientation == ORIENT_HORIZ) 
-        m->paneliconsize = GTK_WIDGET(p->panel->box)->allocation.height
+        m->paneliconsize = p->panel->ah
             - 2* GTK_WIDGET(p->panel->box)->style->ythickness;
     else
-        m->paneliconsize = GTK_WIDGET(p->panel->box)->allocation.width
+        m->paneliconsize = p->panel->aw
             - 2* GTK_WIDGET(p->panel->box)->style->xthickness;
     m->iconsize = 22;       
-    /* menubar */
-    /* holds button to pop menu */
-    m->mainw = gtk_menu_bar_new();
-    //bg_set_transparent_background(m->mainw, p->panel);
-    gtk_widget_set_name(m->mainw, "mainmenu");
-    //gtk_widget_set_name(GTK_WIDGET(m->mainw), "sux-menu");
-    gtk_rc_parse_string(menu_rc);
-    /*
-    if (p->panel->transparent) 
-        gtk_rc_parse_string(transparent_rc);
-    */  
-    gtk_container_set_border_width(GTK_CONTAINER(m->mainw), 0);
-    gtk_container_add(GTK_CONTAINER(p->pwid), m->mainw);
-    gtk_widget_show(m->mainw);
 
-    /* menu button */
-    mi = read_submenu(p, 0);
-    //bg_set_transparent_background(mi, p->panel);
-    if (!mi) {
-        ERR("menu: plugin init failde\n");
+    m->box = gtk_hbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(m->box), 0);
+    gtk_container_add(GTK_CONTAINER(p->pwid), m->box);
+
+    if (!read_submenu(p, FALSE)) {
+        ERR("menu: plugin init failed\n");
         goto error;
     }
-    gtk_menu_shell_append (GTK_MENU_SHELL (m->mainw), mi);
     RET(1);
 
  error:
@@ -360,3 +391,10 @@ plugin_class menu_plugin_class = {
     constructor : menu_constructor,
     destructor  : menu_destructor,
 };
+
+
+/*
+if (level == 0) {
+         
+        } else
+*/
