@@ -33,11 +33,14 @@
 #include "panel.h"
 #include "misc.h"
 #include "plugin.h"
-
+#include "gtkbgbox.h"
 
 //#define DEBUG
 #include "dbg.h"
 
+
+extern panel *p;
+
 /* managed window: all related info that wm holds about its managed windows */
 typedef struct task {
     Window win;
@@ -61,6 +64,7 @@ struct _desk {
     GtkWidget *da;
     GdkPixmap *pix;
     int no, dirty, first;
+    gfloat scalew, scaleh;
     pager *pg;
 };
 
@@ -70,7 +74,7 @@ struct _pager {
     guint desknum;
     guint curdesk;
     int dw, dh;
-    gfloat scalex, scaley;
+    gfloat scalex, scaley, ratio;
     Window *wins;
     int winnum, dirty;
     GHashTable* htable;
@@ -85,15 +89,115 @@ struct _pager {
 //if (t->nws.skip_pager || t->nwwt.desktop /*|| t->nwwt.dock || t->nwwt.splash*/ )
    
 static void pager_rebuild_all(FbEv *ev, pager *pg);
-static void pager_redraw_if_dirty(pager *pg);
+
+static inline void desk_set_dirty_by_win(pager *p, task *t);
+static inline void desk_set_dirty(desk *d);
+static inline void desk_set_dirty_all(pager *pg);
 /*
-static void desk_new_pixmap(desk *d);
 static void desk_clear_pixmap(desk *d);
-static void desk_generate_expose_event(desk *d);
-static gboolean remove_stale_tasks(Window *win, task *t, pager *p);
-static gboolean remove_all_tasks(Window *win, task *t, pager *p);
+static gboolean task_remove_stale(Window *win, task *t, pager *p);
+static gboolean task_remove_all(Window *win, task *t, pager *p);
 */
 
+
+
+
+/*****************************************************************
+ * Task Management Routines                                      *
+ *****************************************************************/
+
+
+/* tell to remove element with zero refcount */
+static gboolean
+task_remove_stale(Window *win, task *t, pager *p)
+{
+    if (t->refcount-- == 0) {
+        desk_set_dirty_by_win(p, t);
+        if (p->focusedtask == t)
+            p->focusedtask = NULL;
+        DBG("del %x\n", t->win);
+        g_free(t);
+        return TRUE;
+    }
+    return FALSE;
+}
+
+/* tell to remove element with zero refcount */
+static gboolean
+task_remove_all(Window *win, task *t, pager *p)
+{
+    g_free(t);
+    return TRUE;
+}
+
+
+static void
+task_get_sizepos(task *t)
+{
+    Window root, junkwin;
+    int dummy, rx, ry;
+    XWindowAttributes win_attributes;
+    
+    ENTER;
+    if (!XGetWindowAttributes(GDK_DISPLAY(), t->win, &win_attributes)) {
+        if (!XGetGeometry (GDK_DISPLAY(), t->win, &root, &t->x, &t->y, &t->w, &t->h,
+                  &dummy, &dummy)) {
+            t->x = t->y = t->w = t->h = 2;
+        }
+            
+    } else {
+        XTranslateCoordinates (GDK_DISPLAY(), t->win, win_attributes.root, 
+              -win_attributes.border_width,
+              -win_attributes.border_width,
+              &rx, &ry, &junkwin);
+        t->x = rx;
+        t->y = ry;
+        t->w = win_attributes.width;
+        t->h = win_attributes.height;
+        DBG("win=0x%x WxH=%dx%d\n", t->win,t->w, t->h);
+    }
+    RET();
+}
+
+
+static void
+task_update_pix(task *t, desk *d)
+{
+    int x, y, w, h;
+    GtkWidget *widget;
+    
+    ENTER;
+    g_return_if_fail(d->pix != NULL);
+    if (!TASK_VISIBLE(t))
+        RET();;
+
+    if (t->desktop < p->desknum &&
+          t->desktop != d->no)
+        RET();
+          
+    x = (gfloat)t->x * d->scalew;
+    y = (gfloat)t->y * d->scaleh;
+    w = (gfloat)t->w * d->scalew;
+    h = (t->nws.shaded) ? 2 : (gfloat)t->h * d->scaleh;
+    if (w < 2 || h < 2)
+        RET();
+    widget = GTK_WIDGET(d->da);
+    gdk_draw_rectangle (d->pix, 
+          (d->pg->focusedtask == t) ?       
+          widget->style->bg_gc[GTK_STATE_SELECTED] :
+          widget->style->bg_gc[GTK_STATE_NORMAL], 
+          TRUE,                         
+          x+1, y+1, w-1, h-1);                
+    gdk_draw_rectangle (d->pix, 
+          (d->pg->focusedtask == t) ?       
+          widget->style->fg_gc[GTK_STATE_SELECTED] :
+          widget->style->fg_gc[GTK_STATE_NORMAL], 
+          FALSE,
+          x, y, w, h);
+    RET();    
+}
+
+
 /*****************************************************************
  * Desk Functions                                                *
  *****************************************************************/
@@ -119,31 +223,37 @@ desk_clear_pixmap(desk *d)
     RET();
 }
 
+
+
 static inline void
-desk_set_dirty(pager *p, task *t) 
+desk_set_dirty(desk *d)
+{
+    ENTER;
+    d->dirty = 1;
+    gtk_widget_queue_draw(d->da);
+    RET();
+}
+
+static inline void
+desk_set_dirty_all(pager *pg)
+{
+    int i;
+    ENTER;
+    for (i = 0; i < pg->desknum; i++)
+        desk_set_dirty(pg->desks[i]);
+    RET();
+}
+
+static inline void
+desk_set_dirty_by_win(pager *p, task *t) 
 {
     ENTER;
     if (t->nws.skip_pager || t->nwwt.desktop /*|| t->nwwt.dock || t->nwwt.splash*/ )
         RET();
-    if (t->desktop < p->desknum)
-        p->desks[t->desktop]->dirty = 1;
+    if (t->desktop < p->desknum) 
+        desk_set_dirty(p->desks[t->desktop]);
     else
-        p->dirty = 1;
-    RET();
-}
-
-static void
-desk_generate_expose_event(desk *d)
-{
-    GdkRectangle r;
-    GtkWidget *widget;
-    
-    ENTER;
-    widget = GTK_WIDGET(d->da);                                   
-    r.x = r.y = 0;
-    r.width = widget->allocation.width;
-    r.height = widget->allocation.height;
-    gtk_widget_queue_draw_area (widget, r.x, r.y, r.width, r.height); /* leads to expose event */
+        desk_set_dirty_all(p);
     RET();
 }
 
@@ -152,9 +262,21 @@ static gint
 desk_expose_event (GtkWidget *widget, GdkEventExpose *event, desk *d)
 {
     ENTER;
-    DBG("d=%p  widget=%p d->da=%p d->no=%d d->pix=%p\n",
-          d, widget, d->da, d->no, d->pix);
-   
+    DBG("d->no=%d\n", d->no);
+
+    if (d->dirty) {
+        pager *pg = d->pg;
+        task *t;
+        int j;
+        
+        d->dirty = 0;
+        desk_clear_pixmap(d);
+        for (j = 0; j < pg->winnum; j++) {
+            if (!(t = g_hash_table_lookup(pg->htable, &pg->wins[j])))
+                continue;
+            task_update_pix(t, d);
+        }
+    }
     gdk_draw_drawable(widget->window,
           widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
           d->pix,
@@ -168,9 +290,9 @@ desk_expose_event (GtkWidget *widget, GdkEventExpose *event, desk *d)
 static gint
 desk_configure_event (GtkWidget *widget, GdkEventConfigure *event, desk *d)
 {
+    int w, h;
     ENTER;
-    DBG("d=%p  d->da=%p d->no=%d d->pix=%p\n", 
-          d, d->da, d->no, d->pix);
+    DBG("d->no=%d %dx%d\n", d->no, widget->allocation.width, widget->allocation.height);
     if (d->pix)
         g_object_unref(d->pix);
     
@@ -178,11 +300,23 @@ desk_configure_event (GtkWidget *widget, GdkEventConfigure *event, desk *d)
           widget->allocation.width,
           widget->allocation.height,
           -1);    
-    desk_clear_pixmap(d);
-    d->dirty = 1;
-    pager_redraw_if_dirty(d->pg);
+
+    d->scalew = (gfloat)widget->allocation.height / (gfloat)gdk_screen_height();
+    d->scaleh = (gfloat)widget->allocation.width  / (gfloat)gdk_screen_width();
+    desk_set_dirty(d);
+
+    //request best size
+    if (p->orientation != ORIENT_HORIZ) {
+        w = widget->allocation.width;
+        h = (gfloat) w / d->pg->ratio;
+    } else {
+        h = widget->allocation.height;
+        w = (gfloat) h * d->pg->ratio;
+    }
+    DBG("requesting %dx%d\n", w, h);
+    gtk_widget_set_size_request(widget, w, h);
     
-    RET(TRUE);
+    RET(FALSE);
 }
 
 static gint
@@ -234,33 +368,32 @@ desk_new(pager *pg, int i)
     ENTER;
     g_assert(i < pg->desknum);
     d = pg->desks[i] = g_new0(desk, 1);
-
     d->pg = pg;
-    d->da = gtk_drawing_area_new();
     d->pix = NULL;
     d->dirty = 0;
     d->first = 1;
     d->no = i;
-    gtk_widget_set_size_request(GTK_WIDGET(d->da), pg->dw, pg->dh);
-    gtk_widget_set_events (d->da, GDK_EXPOSURE_MASK
+    
+    d->da = gtk_drawing_area_new();
+    //gtk_widget_set_size_request(GTK_WIDGET(d->da), 10, 10);
+    gtk_box_pack_start(GTK_BOX(pg->box), d->da, TRUE, TRUE, 0);
+    gtk_widget_add_events (d->da, GDK_EXPOSURE_MASK
           | GDK_BUTTON_PRESS_MASK
           | GDK_BUTTON_RELEASE_MASK);
-          
-    gtk_box_pack_start(GTK_BOX(pg->box), d->da, FALSE, TRUE, 0);
-    gtk_widget_show(d->da);
-
     g_signal_connect (G_OBJECT (d->da), "expose_event",
           (GCallback) desk_expose_event, (gpointer)d);
     g_signal_connect (G_OBJECT (d->da), "configure_event",
-          (GCallback) desk_configure_event, (gpointer)d);
-    g_signal_connect (G_OBJECT (d->da), "button_press_event",
-          (GCallback) desk_button_press_event, (gpointer)d);
-    /*
-    g_signal_connect (G_OBJECT (d->da), "button_release_event",
-    (GCallback) desk_button_release_event, (gpointer)d);
-    */
+          (GCallback) desk_configure_event, (gpointer)d);    
     g_signal_connect (G_OBJECT (d->da), "scroll-event",
-          (GCallback) desk_scroll_event, (gpointer)d);       
+          (GCallback) desk_scroll_event, (gpointer)d);
+    g_signal_connect (G_OBJECT (d->da), "button_press_event",
+         (GCallback) desk_button_press_event, (gpointer)d);
+    //g_signal_connect (G_OBJECT (d->da), "button_release_event",
+    //     (GCallback) desk_button_release_event, (gpointer)d);
+    gtk_widget_show(d->da);
+    DBG("before pack\n");
+
+    DBG("after show\n");
     RET();
 }
 
@@ -281,63 +414,6 @@ desk_free(pager *pg, int i)
 }
 
 
-/*****************************************************************
- * Task Management Routines                                      *
- *****************************************************************/
-
-
-/* tell to remove element with zero refcount */
-static gboolean
-remove_stale_tasks(Window *win, task *t, pager *p)
-{
-    if (t->refcount-- == 0) {
-        desk_set_dirty(p, t);
-        if (p->focusedtask == t)
-            p->focusedtask = NULL;
-        DBG("del %x\n", t->win);
-        g_free(t);
-        return TRUE;
-    }
-    return FALSE;
-}
-
-/* tell to remove element with zero refcount */
-static gboolean
-remove_all_tasks(Window *win, task *t, pager *p)
-{
-    g_free(t);
-    return TRUE;
-}
-
-
-static void
-get_sizepos(task *t)
-{
-    Window root, junkwin;
-    int dummy, rx, ry;
-    XWindowAttributes win_attributes;
-    
-    ENTER;
-    if (!XGetWindowAttributes(GDK_DISPLAY(), t->win, &win_attributes)) {
-        if (!XGetGeometry (GDK_DISPLAY(), t->win, &root, &t->x, &t->y, &t->w, &t->h,
-                  &dummy, &dummy)) {
-            t->x = t->y = t->w = t->h = 2;
-        }
-            
-    } else {
-        XTranslateCoordinates (GDK_DISPLAY(), t->win, win_attributes.root, 
-              -win_attributes.border_width,
-              -win_attributes.border_width,
-              &rx, &ry, &junkwin);
-        t->x = rx;
-        t->y = ry;
-        t->w = win_attributes.width;
-        t->h = win_attributes.height;
-        DBG("win=0x%x WxH=%dx%d\n", t->win,t->w, t->h);
-    }
-    RET();
-}
-
 /*****************************************************************
  * Netwm/WM Interclient Communication                            *
  *****************************************************************/
@@ -355,19 +431,18 @@ do_net_active_window(FbEv *ev, pager *p)
         t = g_hash_table_lookup(p->htable, fwin);
         if (t != p->focusedtask) {
             if (p->focusedtask)
-                desk_set_dirty(p, p->focusedtask);
+                desk_set_dirty_by_win(p, p->focusedtask);
             p->focusedtask = t;
             if (t)
-                desk_set_dirty(p, t);
+                desk_set_dirty_by_win(p, t);
         }
         XFree(fwin);
     } else {
         if (p->focusedtask) {
-            desk_set_dirty(p, p->focusedtask);
+            desk_set_dirty_by_win(p, p->focusedtask);
             p->focusedtask = NULL;
         }
     }
-    pager_redraw_if_dirty(p);
     RET();
 }
 
@@ -375,12 +450,11 @@ static void
 do_net_current_desktop(FbEv *ev, pager *p)
 {
     ENTER;
-    p->desks[p->curdesk]->dirty = 1;
+    desk_set_dirty(p->desks[p->curdesk]);
     p->curdesk =  get_net_current_desktop ();
     if (p->curdesk >= p->desknum)
         p->curdesk = 0;
-    p->desks[p->curdesk]->dirty = 1;
-    pager_redraw_if_dirty(p);
+    desk_set_dirty(p->desks[p->curdesk]);
     RET();
 }
 
@@ -394,7 +468,8 @@ do_net_client_list_stacking(FbEv *ev, pager *p)
     ENTER;
     if (p->wins)
         XFree(p->wins);
-    p->wins = get_xaproperty (GDK_ROOT_WINDOW(), a_NET_CLIENT_LIST_STACKING, XA_WINDOW, &p->winnum);
+    p->wins = get_xaproperty (GDK_ROOT_WINDOW(), a_NET_CLIENT_LIST_STACKING,
+          XA_WINDOW, &p->winnum);
     if (!p->wins || !p->winnum)
         RET();
 
@@ -404,7 +479,7 @@ do_net_client_list_stacking(FbEv *ev, pager *p)
             t->refcount++;
             if (t->stacking != i) {
                 t->stacking = i;
-                desk_set_dirty(p, t);
+                desk_set_dirty_by_win(p, t);
             }
         } else {
             t = g_new0(task, 1);
@@ -414,17 +489,16 @@ do_net_client_list_stacking(FbEv *ev, pager *p)
             t->desktop = get_net_wm_desktop(t->win);
             get_net_wm_state(t->win, &t->nws);
             get_net_wm_window_type(t->win, &t->nwwt);
-            get_sizepos(t);
-            if (!gdk_window_lookup(t->win))
+            task_get_sizepos(t);
+            if (!FBPANEL_WIN(t->win))
                 XSelectInput (GDK_DISPLAY(), t->win, PropertyChangeMask | StructureNotifyMask);
             g_hash_table_insert(p->htable, &t->win, t);
             DBG("add %x\n", t->win);
-            desk_set_dirty(p, t);
+            desk_set_dirty_by_win(p, t);
         }
     }
     /* pass throu hash table and delete stale windows */
-    g_hash_table_foreach_remove(p->htable, (GHRFunc) remove_stale_tasks, (gpointer)p);
-    pager_redraw_if_dirty(p);
+    g_hash_table_foreach_remove(p->htable, (GHRFunc) task_remove_stale, (gpointer)p);
     RET();
 }
 
@@ -443,8 +517,7 @@ pager_unmapnotify(pager *p, XEvent *ev)
     DBG("pager_unmapnotify: win=0x%x\n", win);
     RET();
     t->ws = WithdrawnState;
-    desk_set_dirty(p, t);
-    pager_redraw_if_dirty(p);
+    desk_set_dirty_by_win(p, t);
     RET();
 }
 */
@@ -459,9 +532,8 @@ pager_configurenotify(pager *p, XEvent *ev)
     if (!(t = g_hash_table_lookup(p->htable, &win))) 
         RET();
     DBG("win=0x%x\n", win);
-    get_sizepos(t);
-    desk_set_dirty(p, t);
-    pager_redraw_if_dirty(p);
+    task_get_sizepos(t);
+    desk_set_dirty_by_win(p, t);
     RET();
 }
 
@@ -486,13 +558,12 @@ pager_propertynotify(pager *p, XEvent *ev)
         get_net_wm_state(t->win, &t->nws);
     } else if (at == a_NET_WM_DESKTOP) {	   
         DBG("event=NET_WM_DESKTOP\n");
-        desk_set_dirty(p, t); // to clean up desks where this task was 
+        desk_set_dirty_by_win(p, t); // to clean up desks where this task was 
         t->desktop = get_net_wm_desktop(t->win);
     } else {
         RET();
     }
-    desk_set_dirty(p, t);
-    pager_redraw_if_dirty(p);        
+    desk_set_dirty_by_win(p, t);
     RET();
 }
 
@@ -508,66 +579,6 @@ pager_event_filter( XEvent *xev, GdkEvent *event, pager *pg)
 }
 
 
-static void
-pager_redraw_if_dirty(pager *pg)
-{
-    int i, j, x, y, w, h;
-    GtkWidget *widget;
-    task *t;
-    desk *d;
-
-    ENTER;
-    if (pg->dirty)
-        for (i = 0; i < pg->desknum; i++) {
-            pg->desks[i]->dirty = 1;
-        }
-    for (i = 0; i < pg->desknum; i++) 
-        if (pg->desks[i]->dirty) {
-            desk_clear_pixmap(pg->desks[i]);
-        }
-
-  
-    for (j = 0; j < pg->winnum; j++) {
-        if (!(t = g_hash_table_lookup(pg->htable, &pg->wins[j])))
-            continue;
-        if (!TASK_VISIBLE(t))
-            continue;
-
-        x = (gfloat)t->x * pg->scalex;
-        y = (gfloat)t->y * pg->scaley;
-        w = (gfloat)t->w * pg->scalex;
-        h = (t->nws.shaded) ? 2 : (gfloat)t->h * pg->scaley;
-        if (w < 2 || h < 2)
-            continue;
-        i = (t->desktop < pg->desknum) ? t->desktop : 0;
-        do {
-            d = pg->desks[i];
-            if (!d->dirty || !d->pix) 
-                continue;
-            widget = GTK_WIDGET(d->da); 
-            gdk_draw_rectangle (d->pix, 
-                  (pg->focusedtask == t) ?       
-                  widget->style->bg_gc[GTK_STATE_SELECTED] :
-                  widget->style->bg_gc[GTK_STATE_NORMAL], 
-                  TRUE,                         
-                  x+1, y+1, w-1, h-1);                
-            gdk_draw_rectangle (d->pix, 
-                  (pg->focusedtask == t) ?       
-                  widget->style->fg_gc[GTK_STATE_SELECTED] :
-                  widget->style->fg_gc[GTK_STATE_NORMAL], 
-                  FALSE,
-                  x, y, w, h);
-        } while ((t->desktop > pg->desknum) && ++i < pg->desknum);
-    }
-
-    for (i = 0; i < pg->desknum; i++) 
-        if (pg->desks[i]->dirty) {
-            desk_generate_expose_event(pg->desks[i]);
-            pg->desks[i]->dirty = 0;
-        }
-    pg->dirty = 0;
-    RET();
-}
 
 
 
@@ -595,77 +606,53 @@ pager_rebuild_all(FbEv *ev, pager *pg)
     dif = pg->desknum - desknum;
 
     if (dif == 0)
-        RET();;
+        RET();
 
     if (dif < 0) {
         /* if desktops were deleted then delete their maps also */
-        for (i = desknum; i > pg->desknum; i--) 
-            desk_free(pg, i-1);
-        //pg->desks = g_renew(desk, pg->desks, pg->desknum);
+        for (i = pg->desknum; i < desknum; i++) 
+            desk_free(pg, i);
     } else {
-        //pg->desks = g_renew(desk, pg->desks, pg->desknum);
         for (i = desknum; i < pg->desknum; i++) 
             desk_new(pg, i);
     }
-    pg->dirty = 1;
-    RET();
-}
-/* Upon realize and every resize creates a new backing pixmap of the appropriate size */
-static void
-pager_map_event (GtkWidget *widget, pager *p)
-{
-    ENTER;
-    do_net_client_list_stacking(NULL, p);
-    do_net_active_window(fbev, p);
+    do_net_client_list_stacking(NULL, pg);
     RET();
 }
 
+
 static int
-pager_constructor(plugin *p)
+pager_constructor(plugin *plug)
 {
-
-    line s;
     pager *pg;
     
     ENTER;
-    s.len = 256;
-    while (get_line(p->fp, &s) != LINE_BLOCK_END) {
-        ERR("pager: illegal in this context %s\n", s.str);
-        RET(0);
-    }
     pg = g_new0(pager, 1);
     g_return_val_if_fail(pg != NULL, 0);
-    p->priv = pg;
+    plug->priv = pg;
 
     pg->htable = g_hash_table_new (g_int_hash, g_int_equal);
-    // pager is wrapped into eventbox to ensure non-transparent bg
-    pg->eb = gtk_event_box_new();
-    gtk_container_set_border_width (GTK_CONTAINER (pg->eb), 0);
-    gtk_widget_show(pg->eb);
-    pg->box = p->panel->my_box_new(TRUE, 1);
-    gtk_container_set_border_width (GTK_CONTAINER (pg->box), 1);
+
+    pg->box = plug->panel->my_box_new(TRUE, 1);
+    gtk_container_set_border_width (GTK_CONTAINER (pg->box), 0);
     gtk_widget_show(pg->box);
-    g_signal_connect (G_OBJECT (pg->box), "map",
-          (GCallback) pager_map_event, (gpointer)pg);
-    gtk_container_add(GTK_CONTAINER(pg->eb), pg->box);
-    gtk_container_add(GTK_CONTAINER(p->pwid), pg->eb);
-    if (p->panel->orientation == ORIENT_HORIZ) {
-        pg->dh = p->panel->ah - 2;
-        pg->dw = (float)pg->dh * (float)gdk_screen_width() / (float) gdk_screen_height();
-    } else {
-        pg->dw = p->panel->aw - 2;
-        pg->dh = (float)pg->dw * (float)gdk_screen_height() / (float) gdk_screen_width();
-    }
+  
+    gtk_bgbox_set_background(plug->pwid, BG_STYLE, 0, 0);
+    gtk_container_set_border_width (GTK_CONTAINER (plug->pwid), 1);
+    gtk_container_add(GTK_CONTAINER(plug->pwid), pg->box);
+    pg->eb = pg->box;
+
+    pg->ratio = (gfloat)gdk_screen_width() / (gfloat)gdk_screen_height();
     pg->scaley = (gfloat)pg->dh / (gfloat)gdk_screen_height();
     pg->scalex = (gfloat)pg->dw / (gfloat)gdk_screen_width();
 
-    //pg->htable = g_hash_table_new (g_int_hash, g_int_equal);
+
     pager_rebuild_all(fbev, pg);
-    do_net_current_desktop(fbev, pg);
-    do_net_client_list_stacking(fbev, pg);
+    //do_net_current_desktop(fbev, pg);
+    //do_net_client_list_stacking(fbev, pg);
     
-    pager_redraw_if_dirty(pg);
     gdk_window_add_filter(NULL, (GdkFilterFunc)pager_event_filter, pg );
+  
     g_signal_connect (G_OBJECT (fbev), "current_desktop",
           G_CALLBACK (do_net_current_desktop), (gpointer) pg);
     g_signal_connect (G_OBJECT (fbev), "active_window",
@@ -674,7 +661,6 @@ pager_constructor(plugin *p)
           G_CALLBACK (pager_rebuild_all), (gpointer) pg);
     g_signal_connect (G_OBJECT (fbev), "client_list_stacking",
           G_CALLBACK (do_net_client_list_stacking), (gpointer) pg);
-    
     RET(1);
 }
 
@@ -692,7 +678,7 @@ pager_destructor(plugin *p)
     while (--pg->desknum) {
         desk_free(pg, pg->desknum);
     }
-    g_hash_table_foreach_remove(pg->htable, (GHRFunc) remove_all_tasks, (gpointer)pg);
+    g_hash_table_foreach_remove(pg->htable, (GHRFunc) task_remove_all, (gpointer)pg);
     g_hash_table_destroy(pg->htable);
     gtk_widget_destroy(pg->eb);
     g_free(pg);
