@@ -79,7 +79,6 @@ tray_destructor(plugin *p)
     if (tr->tray_manager)
         g_object_unref (G_OBJECT (tr->tray_manager));
     fixed_tip_hide ();
-    gtk_widget_destroy(tr->mainw);
     g_free(tr);
     RET();
 }
@@ -93,7 +92,7 @@ tray_constructor(plugin *p)
     line s;
     tray *tr;
     GdkScreen *screen;
-    GtkWidget *ali;
+    GtkWidget *frame;
     
     ENTER;
     s.len = 256;
@@ -108,17 +107,15 @@ tray_constructor(plugin *p)
     p->priv = tr;
     tr->plug = p;
 
-    //tr->mainw = gtk_event_box_new();
-    tr->mainw = gtk_bgbox_new();
-  
-    ali = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
+    frame = gtk_frame_new(NULL);
+    //gtk_container_set_border_width(GTK_CONTAINER(frame), 0);
+    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
+    gtk_container_add(GTK_CONTAINER(p->pwid), frame);    
     tr->box = p->panel->my_box_new(FALSE, 2);
-    gtk_container_add (GTK_CONTAINER (ali), tr->box);
-    gtk_container_add (GTK_CONTAINER (tr->mainw), ali);
-    gtk_container_add(GTK_CONTAINER(p->pwid), tr->mainw);
-    if (p->panel->transparent) 
-        gtk_bgbox_set_background(tr->mainw, BG_ROOT, p->panel->tintcolor, p->panel->alpha);        
-
+    gtk_container_add(GTK_CONTAINER(frame), tr->box);
+    
+    gtk_bgbox_set_background(p->pwid, BG_STYLE, 0, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(p->pwid), 1);
     screen = gtk_widget_get_screen (GTK_WIDGET (p->panel->topgwin));
     
     if (egg_tray_manager_check_running(screen)) {
@@ -139,7 +136,7 @@ tray_constructor(plugin *p)
     g_signal_connect (tr->tray_manager, "message_cancelled",
           G_CALLBACK (message_cancelled), tr->box);
     
-    
+    gtk_widget_show_all(frame);
     RET(1);
 
 }
